/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <media_content.h>
#include <app_debug.h>
#include <app_contents.h>
#include <app_media.h>
#include <gridmgr.h>
#include <inputmgr.h>
#include <viewmgr.h>

#include "define.h"
#include "view.h"
#include "data/mediadata.h"
#include "grid/grid.h"
#include "util/util.h"

#define STR_GENRE_NAME "Genre"

static char *_media_text_get(void *data, Evas_Object *obj, const char *part)
{
	app_media *am;
	app_media_info *info;

	if (!data)
		return NULL;

	am = data;

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get media info");
		return NULL;
	}

	return strdup(info->title);
}

static Evas_Object *_media_content_get(void *data, Evas_Object *obj,
			const char *part)
{
	Evas_Object *eo;
	app_media *am;
	app_media_info *info;
	struct color_data bg;

	if (!data)
		return NULL;

	am = data;

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get media info");
		return NULL;
	}

	eo = NULL;
	if (!strcmp(part, PART_ELM_SWALLOW_THUMBNAIL)) {
		eo = util_add_image(obj, info->thumbnail_path);
		if (!eo) {
			_ERR("failed to create image object");
			return NULL;
		}
	} else if (!strcmp(part, PART_ELM_SWALLOW_FAVORITE)) {
		if (!info->favorite)
			return NULL;

		eo = util_add_image(obj, IMAGE_THUMBNAIL_FAVORITE);
		if (!eo) {
			_ERR("failed to create image object");
			return NULL;
		}
	} else if (!strcmp(part, PART_ELM_SWALLOW_TEXTBG)) {
		eo = evas_object_rectangle_add(obj);
		if (!eo) {
			_ERR("failed to create rectangle object");
			return NULL;
		}

		app_contents_get_color(info->title, NULL, &bg);
		evas_object_color_set(eo, bg.r, bg.g, bg.b, bg.a);
	}

	if (eo)
		evas_object_show(eo);

	return eo;
}

static char *_group_text_get(void *data, Evas_Object *obj, const char *part)
{
	struct group_info *gi;
	char buf[32];
	int cnt;

	if (!data)
		return NULL;

	gi = data;

	if (!strcmp(part, PART_ELM_TEXT_TITLE))
		return strdup(gi->name);
	else if (!strcmp(part, PART_ELM_TEXT_COUNT)) {
		cnt = eina_list_count(gi->list);
		snprintf(buf, sizeof(buf), "%d", cnt);
		return strdup(buf);
	}

	return NULL;
}

static Evas_Object *_group_content_get(void *data, Evas_Object *obj,
			const char *part)
{
	Evas_Object *eo;
	struct group_info *gi;
	struct color_data bg;

	if (!data)
		return NULL;

	gi = data;

	eo = NULL;
	if (!strcmp(part, PART_ELM_SWALLOW_BG)) {
		eo = evas_object_rectangle_add(obj);
		if (!eo) {
			_ERR("failed to create rectangle object");
			return NULL;
		}

		app_contents_get_color(gi->name, &bg, NULL);
		evas_object_color_set(eo, bg.r, bg.g, bg.b, bg.a);

		evas_object_show(eo);
	}

	return eo;
}

static Eina_List *_get_name_list(struct mediadata *md)
{
	if (!md) {
		_ERR("invalid argument");
		return NULL;
	}

	return mediadata_get_list(md, E_LIST_NAME);
}

static Eina_List *_get_genre_list(struct mediadata *md)
{
	Eina_List *list;
	struct group_info *gi;

	if (!md) {
		_ERR("invalid argument");
		return NULL;
	}

	gi = calloc(1, sizeof(*gi));
	if (!gi) {
		_ERR("failed to allocate memory");
		return NULL;
	}

	gi->name = STR_GENRE_NAME;
	gi->list = mediadata_get_list(md, E_LIST_GENRE);

	list = NULL;
	list = eina_list_append(list, gi);

	return list;
}

static Eina_List *_get_date_list(struct mediadata *md)
{
	if (!md) {
		_ERR("invalid argument");
		return NULL;
	}

	return mediadata_get_list(md, E_LIST_DATE);
}

static void _free_media_list(Eina_List *list)
{
	mediadata_free_list(list);
}

static void _free_group_list(Eina_List *list)
{
	Eina_List *l;
	struct group_info *gi;

	EINA_LIST_FOREACH(list, l, gi) {
		mediadata_free_list(gi->list);
		free(gi);
	}
}

static void _key_down_cb(void *data, Elm_Object_Item *it,
			Evas_Event_Key_Down *ev)
{
	struct view_update_data vdata;
	struct mediadata *md;
	app_media *am;

	if (!data || !ev) {
		_ERR("invalid argument");
		return;
	}

	md = data;

	if (!strcmp(ev->keyname, KEY_MENU) ||
		!strcmp(ev->keyname, KEY_MENU_REMOTE)) {
		am = elm_object_item_data_get(it);
		if (!am) {
			_ERR("failed to get app media");
			return;
		}

		vdata.list = mediadata_get_medialist(md);
		vdata.index = util_get_media_index(vdata.list, am);

		viewmgr_update_view(VIEW_ACTION_MENU, UPDATE_CONTENT, &vdata);
		viewmgr_show_view(VIEW_ACTION_MENU);
	}
}

static void _selected_cb(void *data, Elm_Object_Item *it)
{
	struct view_update_data vdata;
	struct mediadata *md;
	app_media *am;

	if (!data || !it) {
		_ERR("invalid argument");
		return;
	}

	md = data;

	am = elm_object_item_data_get(it);
	if (!am) {
		_ERR("failed to get app media");
		return;
	}

	vdata.list = mediadata_get_medialist(md);
	vdata.index = util_get_media_index(vdata.list, am);

	viewmgr_update_view(VIEW_VIEWER, UPDATE_CONTENT, &vdata);
	viewmgr_push_view(VIEW_VIEWER);
}

static struct grid_class _gclass[] = {
	[E_MOVIE_NAME] = {
		.item_style = STYLE_MOVIE_NAME,
		.text_get = _media_text_get,
		.content_get = _media_content_get
	},
	[E_MOVIE_GENRE] = {
		.item_style = STYLE_MOVIE_GENRE,
		.text_get = _group_text_get,
		.content_get = _group_content_get
	},
	[E_MOVIE_DATE] = {
		.item_style = STYLE_MOVIE_NAME,
		.text_get = _media_text_get,
		.content_get = _media_content_get
	},
	/* Other view mode class will be added later */
};

static struct grid_data _gdata[] = {
	[E_MOVIE_NAME] = {
		.item_x = MOVIE_NAME_ITEM_X,
		.item_y = MOVIE_NAME_ITEM_Y,
		.item_num = MOVIE_NAME_ITEM_NUM,
		.grid_padding = MOVIE_NAME_GRID_PADDING,
		.gclass = &_gclass[E_MOVIE_NAME],
		.get_item_list = _get_name_list,
		.free_item_list = _free_media_list,
		.key_down_cb = _key_down_cb,
		.selected_cb = _selected_cb
	},
	[E_MOVIE_GENRE] = {
		.item_x = MOVIE_GENRE_ITEM_X,
		.item_y = MOVIE_GENRE_ITEM_Y,
		.item_num = MOVIE_GENRE_ITEM_NUM,
		.grid_padding = MOVIE_GENRE_GRID_PADDING,
		.gclass = &_gclass[E_MOVIE_GENRE],
		.get_item_list = _get_genre_list,
		.free_item_list = _free_group_list
	},
	[E_MOVIE_DATE] = {
		.item_x = MOVIE_NAME_ITEM_X,
		.item_y = MOVIE_NAME_ITEM_Y,
		.item_num = MOVIE_NAME_ITEM_NUM,
		.grid_padding = MOVIE_NAME_GRID_PADDING,
		.gclass = &_gclass[E_MOVIE_DATE],
		.get_item_list = _get_date_list,
		.free_item_list = _free_media_list
	},
	/* Other view mode data will be added later */
};

struct grid_data *get_movie_grid_data(int type)
{
	if (type < 0 || type >= E_MOVIE_MAX) {
		_ERR("invalid argument");
		return NULL;
	}

	return &_gdata[type];
}
