/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <media_content.h>
#include <app_debug.h>
#include <app_contents.h>
#include <app_media.h>
#include <gridmgr.h>
#include <inputmgr.h>
#include <viewmgr.h>

#include "define.h"
#include "view.h"
#include "data/mediadata.h"
#include "grid/grid.h"
#include "util/util.h"

static char *_grid_text_get(void *data, Evas_Object *obj, const char *part)
{
	app_media *am;
	app_media_info *info;

	if (!data)
		return NULL;

	am = data;

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get media info");
		return NULL;
	}

	if (!strcmp(part, PART_ELM_TEXT_TITLE))
		return strdup(info->title);
	else if (!strcmp(part, PART_ELM_TEXT_ARTIST))
		return strdup(info->audio->artist);
	else if (!strcmp(part, PART_ELM_TEXT_ALBUM))
		return strdup(info->audio->album);

	return NULL;
}

static Evas_Object *_grid_content_get(void *data,
			Evas_Object *obj, const char *part)
{
	Evas_Object *eo;
	app_media *am;
	app_media_info *info;

	if (!data)
		return NULL;

	am = data;

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get media info");
		return NULL;
	}

	eo = NULL;
	if (!strcmp(part, PART_ELM_SWALLOW_THUMBNAIL)) {
		eo = util_add_image(obj, info->thumbnail_path);
		if (!eo) {
			_ERR("failed to create image object");
			return NULL;
		}
	} else if (!strcmp(part, PART_ELM_SWALLOW_FAVORITE)) {
		if (!info->favorite)
			return NULL;

		eo = util_add_image(obj, IMAGE_THUMBNAIL_FAVORITE);
		if (!eo) {
			_ERR("failed to create image object");
			return NULL;
		}
	}

	if (eo)
		evas_object_show(eo);

	return eo;
}

static Eina_List *_get_item_list(struct mediadata *md)
{
	if (!md) {
		_ERR("invalid argument");
		return NULL;
	}

	return mediadata_get_list(md, E_LIST_NAME);
}

static void _free_item_list(Eina_List *list)
{
	mediadata_free_list(list);
}

static void _key_down_cb(void *data, Elm_Object_Item *it,
			Evas_Event_Key_Down *ev)
{
	struct view_update_data vdata;
	struct mediadata *md;
	app_media *am;

	if (!data || !ev) {
		_ERR("invalid argument");
		return;
	}

	md = data;

	if (!strcmp(ev->keyname, KEY_MENU) ||
		!strcmp(ev->keyname, KEY_MENU_REMOTE)) {
		am = elm_object_item_data_get(it);
		if (!am) {
			_ERR("failed to get app media");
			return;
		}

		vdata.list = mediadata_get_medialist(md);
		vdata.index = util_get_media_index(vdata.list, am);

		viewmgr_update_view(VIEW_ACTION_MENU, UPDATE_CONTENT, &vdata);
		viewmgr_show_view(VIEW_ACTION_MENU);
	}
}

static void _selected_cb(void *data, Elm_Object_Item *it)
{
	struct view_update_data vdata;
	struct mediadata *md;
	app_media *am;

	if (!data || !it) {
		_ERR("invalid argument");
		return;
	}

	md = data;

	am = elm_object_item_data_get(it);
	if (!am) {
		_ERR("failed to get app media");
		return;
	}

	vdata.list = mediadata_get_medialist(md);
	vdata.index = util_get_media_index(vdata.list, am);

	viewmgr_update_view(VIEW_MPLAYER, UPDATE_CONTENT, &vdata);
	viewmgr_push_view(VIEW_MPLAYER);
}

static struct grid_class _gclass[] = {
	[E_MUSIC_SONG] = {
		.item_style = STYLE_MUSIC_SONG,
		.text_get = _grid_text_get,
		.content_get = _grid_content_get
	},
	/* Other view mode class will be added later */
};

static struct grid_data _gdata[] = {
	[E_MUSIC_SONG] = {
		.item_x = MUSIC_SONG_ITEM_X,
		.item_y = MUSIC_SONG_ITEM_Y,
		.item_num = MUSIC_SONG_ITEM_NUM,
		.grid_padding = MUSIC_SONG_GRID_PADDING,
		.gclass = &_gclass[E_MUSIC_SONG],
		.get_item_list = _get_item_list,
		.free_item_list = _free_item_list,
		.key_down_cb = _key_down_cb,
		.selected_cb = _selected_cb
	},
	/* Other view mode data will be added later */
};

struct grid_data *get_music_grid_data(int type)
{
	if (type < 0 || type >= E_MUSIC_MAX) {
		_ERR("invalid argument");
		return NULL;
	}

	return &_gdata[E_MUSIC_SONG];
}
